@@ -1,12 +1,12 @@
-from collections import OrderedDict
 import types
-from typing import Any, Callable, List, Union, Tuple
+from collections import OrderedDict
+from typing import Any, Callable, List, Tuple, Union
 
 import numpy as np
 import pandas as pd
 import statsmodels.api as sm
-from statsmodels.regression.linear_model import OLS
 from numba import njit
+from statsmodels.regression.linear_model import OLS
 
 from qubx.pandaz.utils import check_frame_columns, continuous_periods, has_columns, ohlc_resample, scols, srows
 from qubx.utils.misc import Struct
@@ -622,15 +622,16 @@ def series_halflife(series: pd.Series | np.ndarray) -> float:
     return np.ceil(-np.log(2) / reg.params[0])
 
 
-def rolling_std_with_mean(x: pd.Series, mean: float | pd.Series, window: int):
+def rolling_zscore(x: pd.Series, window: int, std_window: int | None = None):
     """
-    Calculates rolling standard deviation for data from x and already calculated mean series
+    Calculates rolling z-score for data from x
     :param x: series data
-    :param mean: calculated mean
     :param window: window
-    :return: rolling standard deviation
+    :return: rolling z-score
     """
-    return np.sqrt((((x - mean) ** 2).rolling(window=window).sum() / (window - 1)))
+    if std_window is None:
+        std_window = window
+    return (x - x.rolling(window=window).mean()) / x.rolling(window=std_window).std(ddof=0)
 
 
 def bollinger(x: pd.Series, window=14, nstd=2, mean="sma") -> pd.DataFrame:
@@ -645,7 +646,7 @@ def bollinger(x: pd.Series, window=14, nstd=2, mean="sma") -> pd.DataFrame:
     :return: mean, upper and lower bands
     """
     rolling_mean = smooth(x, mean, window)
-    rolling_std = rolling_std_with_mean(x, rolling_mean, window)
+    rolling_std = x.rolling(window=window).std()
 
     upper_band = rolling_mean + (rolling_std * nstd)
     lower_band = rolling_mean - (rolling_std * nstd)
@@ -1326,11 +1327,11 @@ def rolling_rank(x, period, pctls=(25, 50, 75)):
         raise ValueError(f"Period {period} exceeds number of data records {len(x)} ")
 
     if isinstance(x, pd.DataFrame):
-        z = pd.DataFrame.from_dict({c: rolling_rank(s, period, pctls) for c, s in x.iteritems()})
+        z = pd.DataFrame.from_dict({c: rolling_rank(s, period, pctls) for c, s in x.items()})
     elif isinstance(x, pd.Series):
-        z = pd.Series(_rolling_rank(x.values, period, pctls), x.index, name=x.name)
+        z = pd.Series(_rolling_rank(np.ascontiguousarray(x.values), period, pctls), x.index, name=x.name)
     else:
-        z = _rolling_rank(x.values, period, pctls)
+        z = _rolling_rank(np.ascontiguousarray(x), period, pctls)
     return z
 
 
@@ -1351,40 +1352,106 @@ def rolling_vwap(ohlc, period):
     return __rollsum((ohlc.volume * ohlc.close), period) / __rollsum(ohlc.volume, period)
 
 
-def fractals(data, nf, actual_time=True, align_with_index=False) -> pd.DataFrame:
+def pivots_highs_lows(
+    high: pd.Series,
+    low: pd.Series,
+    nf_before: int,
+    nf_after: int,
+    index_on_observed_time=True,
+    align_with_index=False,
+    include_actual_time=False,
+) -> pd.DataFrame:
     """
-    Calculates fractals indicator
+    Calculates pivots indicator (pivot highs/lows) for asymmetric lookback/lookahead.
 
-    :param data: OHLC bars series
-    :param nf: fractals lookback/foreahed parameter
-    :param actual_time: if true fractals timestamps at bars where they would be observed
-    :param align_with_index: if true result will be reindexed as input ohlc data
-    :return: pd.DataFrame with U (upper) and L (lower) fractals columns
+    Example:
+    >>> high = pd.Series([1, 2, 3, 4, 5, 6, 5, 4, 3])
+    >>> low = pd.Series([5, 4, 3, 2, 3, 3, 1, 3, 4])
+    >>> pta.pivots_highs_lows(high, low, 3, 2, index_on_observed_time=True, align_with_index=False)
+
+    :param high: high series
+    :param low: low series
+    :param nf_before: number of bars to look backward
+    :param nf_after: number of bars to look forward
+    :param index_on_observed_time: If True, pivot is placed at the bar where it becomes visible
+    :param align_with_index: If True, result is reindexed to input data index
+    :param include_actual_time: If True, include time of bar where pivot occured in result
+    :return: DataFrame with 'U' (upper pivots), 'L' (lower pivots). Result is multi-index if include_actual_time is True
     """
-    check_frame_columns(data, "high", "low")
+    if len(high) != len(low):
+        raise ValueError("High and low series must have the same length !")
 
-    ohlc = scols(data.close.ffill(), data, keys=["A", "ohlc"]).ffill(axis=1)["ohlc"]
-    ru, rd = None, None
-    for i in range(1, nf + 1):
-        ru = scols(
-            ru, (ohlc.high - ohlc.high.shift(i)).rename(f"p{i}"), (ohlc.high - ohlc.high.shift(-i)).rename(f"p_{i}")
-        )
-        rd = scols(rd, (ohlc.low.shift(i) - ohlc.low).rename(f"p{i}"), (ohlc.low.shift(-i) - ohlc.low).rename(f"p_{i}"))
+    if any(high.index != low.index):
+        raise ValueError("High and low series must have the same index !")
 
-    ru, rd = ru.dropna(), rd.dropna()
+    idx = high.index
 
-    upF = pd.Series(+1, ru[((ru > 0).all(axis=1))].index)
-    dwF = pd.Series(-1, rd[((rd > 0).all(axis=1))].index)
+    # For upper fractals: current high must be greater than all highs in window
+    highs_before = pd.concat([high.shift(i) for i in range(1, nf_before + 1)], axis=1)
+    highs_after = pd.concat([high.shift(-i) for i in range(1, nf_after + 1)], axis=1)
+    upper_cond = (highs_before.lt(high, axis=0).all(axis=1)) & (highs_after.lt(high, axis=0).all(axis=1))
+    upF = pd.Series(+1, idx[upper_cond])
 
-    shift_forward = nf if actual_time else 0
-    ht = ohlc.loc[upF.index].reindex(ohlc.index).shift(shift_forward).high
-    lt = ohlc.loc[dwF.index].reindex(ohlc.index).shift(shift_forward).low
+    # For lower fractals: current low must be less than all lows in window
+    lows_before = pd.concat([low.shift(i) for i in range(1, nf_before + 1)], axis=1)
+    lows_after = pd.concat([low.shift(-i) for i in range(1, nf_after + 1)], axis=1)
+    lower_cond = (lows_before.gt(low, axis=0).all(axis=1)) & (lows_after.gt(low, axis=0).all(axis=1))
+    dwF = pd.Series(-1, idx[lower_cond])
+
+    # Fractal is observed only after the 'future' bars, so shift by nf_after if actual_time
+    shift_forward = nf_after if index_on_observed_time else 0
+
+    ht = high.loc[upF.index].reindex(idx).shift(shift_forward)
+    lt = low.loc[dwF.index].reindex(idx).shift(shift_forward)
 
     if not align_with_index:
         ht = ht.dropna()
         lt = lt.dropna()
 
-    return scols(ht, lt, names=["U", "L"])
+    if include_actual_time:
+        _l_times = pd.Series(dwF.index, index=dwF.index).reindex(idx).shift(shift_forward)
+        _h_times = pd.Series(upF.index, index=upF.index).reindex(idx).shift(shift_forward)
+        if not align_with_index:
+            _h_times = _h_times.dropna()
+            _l_times = _l_times.dropna()
+
+        # return pd.DataFrame({"U": ht, "L": lt, "U_time": _h_times, "L_time": _l_times})
+        return scols(
+            pd.DataFrame({"price": ht, "time": _h_times}),
+            pd.DataFrame({"price": lt, "time": _l_times}),
+            keys=["U", "L"],
+        )
+
+    return pd.DataFrame({"U": ht, "L": lt})
+
+
+def fractals(
+    data: pd.DataFrame,
+    nf: int,
+    index_on_observed_time: bool = True,
+    align_with_index: bool = False,
+    include_actual_time=False,
+) -> pd.DataFrame:
+    """
+    Calculates fractals indicator
+
+    :param data: OHLC bars series
+    :param nf: fractals lookback/foreahed parameter
+    :param index_on_observed_time: if true fractals timestamps at bars where they would be observed
+    :param align_with_index: if true result will be reindexed as input ohlc data
+    :param include_actual_time: If True, include time of bar where fractals occured in result
+    :return: pd.DataFrame with U (upper) and L (lower) fractals columns
+    """
+    check_frame_columns(data, "high", "low")
+    return pivots_highs_lows(
+        data["high"],
+        data["low"],
+        nf,
+        nf,
+        index_on_observed_time,
+        align_with_index,
+        include_actual_time=include_actual_time,
+    )
 
 
 @njit
@@ -1413,6 +1480,7 @@ def _jma(x, period, phase, power):
     return x
 
 
+@__wrap_dataframe_decorator
 def jma(x, period, phase=0, power=2):
     """
     Jurik MA (code from https://www.tradingview.com/script/nZuBWW9j-Jurik-Moving-Average/)
@@ -1523,38 +1591,94 @@ def super_trend(
     return scols(longstop_res, shortstop_res, direction, names=["utl", "dtl", "trend"])
 
 
-def choppyness(data, period, upper=61.8, lower=38.2, atr_smoother="sma") -> pd.Series:
+def choppiness(
+    data,
+    period,
+    upper=61.8,
+    lower=38.2,
+    volatility_estimator="t",
+    volume_adjusting=False,
+    identification="strong",
+    with_raw_indicator=False,
+) -> pd.Series:
     """
-    Volatile market leads to false breakouts, and not respecting support/resistance levels (being choppy),
+    Calculate market choppiness index using volatility-based formula.
+
+    Volatile market leads to false breakouts and does not respect support/resistance levels (being choppy).
     We cannot know whether we are in a trend or in a range.
 
-    Values above 61.8% indicate a choppy market that is bound to breakout. We should be ready for some directional.
+    Values above 61.8% indicate a choppy market that is bound to breakout. We should be ready for some directional movement.
     Values below 38.2% indicate a strong trending market that is bound to stabilize.
 
-    :param data: ohlc dataframe
-    :param period: period of lookback
-    :param upper: upper threshold (61.8)
-    :param lower: lower threshold (38.2)
-    :param atr_smoother: used ATR smoother (SMA)
-    :return: series of choppyness indication (True - market is 'choppy', False - market is trend)
-    """
-    check_frame_columns(data, "open", "high", "low", "close")
+    When volatility_estimator="t" (true range), this gives the classical Dreiss choppiness formula.
 
-    xr = data[["open", "high", "low", "close"]]
-    a = atr(xr, period, atr_smoother)
+    Parameters
+    ----------
+    data : pd.DataFrame
+        OHLCV data frame containing 'open', 'high', 'low', 'close', 'volume' columns
+    period : int
+        Lookback period for calculations
+    upper : float, default 61.8
+        Upper threshold - values above indicate choppy market
+    lower : float, default 38.2
+        Lower threshold - values below indicate trending market
+    volatility_estimator : str, default 't'
+        Method used for volatility calculation (see volatility() function)
+    volume_adjusting : bool, default False
+        If True, adjusts volatility by relative volume changes
+    identification : str, default 'strong'
+        Identification method:
+        - 'mid': Identifies three states: choppy (1) when crossing upper threshold, trending (-1) when crossing lower threshold,
+                 and neutral (0) when between thresholds
+        - 'strong': Binary classification focused on trending - returns 1 when entering trending regime (crossing below lower threshold),
+                    0 when exiting trending regime (crossing above lower threshold)
+        - 'weak': Binary classification focused on choppiness - returns 1 when entering choppy regime (crossing above upper threshold),
+                 0 when entering trending regime (crossing below lower threshold)
+    with_raw_indicator : bool, default False
+        If True, returns the raw indicator value instead of the classification
+    Returns
+    -------
+    pd.Series
+        Boolean series where True indicates choppy market and False indicates trending market
+    """
+
+    if volume_adjusting:
+        check_frame_columns(data, "open", "high", "low", "close", "volume")
+        xr = data[["open", "high", "low", "close", "volume"]]
+    else:
+        check_frame_columns(data, "open", "high", "low", "close")
+        xr = data[["open", "high", "low", "close"]]
 
     rng = (
         xr["high"].rolling(window=period, min_periods=period).max()
         - xr["low"].rolling(window=period, min_periods=period).min()
     )
 
-    rs = pd.Series(rolling_sum(column_vector(a.copy()), period).flatten(), a.index)
-    ci = 100 * np.log(rs / rng) * (1 / np.log(period))
+    if volatility_estimator == "atr":
+        a = atr(xr, period, smoother="ema")
+        vol = pd.Series(rolling_sum(column_vector(a.copy()), period).flatten(), a.index).replace(0, np.nan)
+    else:
+        vol = period * volatility(
+            xr, period, method=volatility_estimator, volume_adjusting=volume_adjusting, percentage=False
+        )
+    ci = 100 * (1 / np.log(period)) * np.log(vol / rng)
 
-    f0 = pd.Series(np.nan, ci.index, dtype=bool)
-    f0[ci >= upper] = True
-    f0[ci <= lower] = False
-    return f0.ffill().fillna(False)
+    f0 = pd.Series(np.nan, ci.index, dtype=int)
+    match identification:
+        case "mid":
+            f0[(ci > lower) & (ci < upper)] = 0
+            f0[ci > upper] = 1
+            f0[ci < lower] = -1
+        case "strong":
+            f0[ci > lower] = 1
+            f0[ci < lower] = 0
+        case "weak":
+            f0[ci > upper] = 1
+            f0[ci < lower] = 0
+        case _:
+            raise ValueError(f"Invalid identification: {identification}")
+
+    return f0 if not with_raw_indicator else ci
 
 
 @njit
@@ -1918,10 +2042,7 @@ def qqe_mod(
     res = res.assign(
         # here we code hist bars:
         #  -1 -> silver < 0, +1 -> silver > 0, +2 -> green, -2 -> red
-        code=-1 * (res.silver < 0)
-        + 1 * (res.silver > 0)
-        + 2 * (res.green > 0)
-        - 2 * (res.red < 0)
+        code=-1 * (res.silver < 0) + 1 * (res.silver > 0) + 2 * (res.green > 0) - 2 * (res.red < 0)
     )
 
     return res
@@ -1987,7 +2108,9 @@ def ssl_exits(
         grow_line,
         decl_line,
         m.base_line,
-        names=["exit_long", "exit_short", "grow", "decline", "base"],
+        m.upperk,
+        m.lowerk,
+        names=["exit_long", "exit_short", "grow", "decline", "base", "upper_k", "lower_k"],
     )
 
 
@@ -2098,7 +2221,7 @@ def norm_pdf(x):
 
 @njit
 def lognorm_pdf(x, s):
-    return np.exp(-np.log(x) ** 2 / (2 * s**2)) / (x * s * np.sqrt(2 * np.pi))
+    return np.exp(-(np.log(x) ** 2) / (2 * s**2)) / (x * s * np.sqrt(2 * np.pi))
 
 
 @njit
@@ -2382,7 +2505,6 @@ def find_movements_hilo(
 
                 # check condition
                 if (vL - xL_mi >= threshold) or (l_mv < vL - xL_mi):
-
                     # case when HighLow of a one bar are extreme points, to avoid infinite loop
                     if mx == mi:
                         mi += 1
@@ -2587,3 +2709,224 @@ def smooth(x: pd.Series, smoother: str | Callable[[pd.Series, Any], pd.Series],
     x_sm = f_sm(x, *args, **kwargs)
 
     return x_sm if isinstance(x_sm, pd.Series) else pd.Series(x_sm.flatten(), index=x.index)
+
+
+def volatility(
+    data: pd.DataFrame,
+    period: int,
+    method: str = "c",
+    volume_adjusting: bool = False,
+    volume_adjustment_method: str = "normal",
+    percentage: bool = True,
+) -> pd.Series:
+    """
+    Calculate historical volatility (as standard deviation) using various estimation methods.
+
+    Parameters
+    ----------
+    data : pd.DataFrame
+        OHLCV data frame containing 'open', 'high', 'low', 'close', 'volume' columns
+    period : int
+        Rolling window size for volatility calculation
+    method : str, default 'c'
+        Volatility estimation method:
+        - 't': True range volatility
+        - 'te': True range with ema smoothing
+        - 'c': Classical close-to-close volatility
+        - 'p': Parkinson high-low volatility
+        - 'rs': Rogers-Satchell volatility
+        - 'gk': Garman-Klass volatility
+        - 'yz': Yang-Zhang volatility (combines overnight and trading volatility)
+    volume_adjusting : bool, default False
+        If True, adjusts volatility by relative volume changes
+    volume_adjustment_method : str, default 'normal'
+        Volume adjustment method:
+        - 'normal': Normal volume adjustment: f_t = v_t / v_{t-1}
+        - 'entropy': Entropy volume adjustment: f_t = v_t / sma(v_t, period)
+    percentage : bool, default True
+        If True, returns volatility in percentage terms, otherwise in price terms
+
+    Returns
+    -------
+    pd.Series
+        Volatility estimates for each period
+    """
+    if volume_adjusting:
+        check_frame_columns(data, "open", "high", "low", "close", "volume")
+        xr = data[["open", "high", "low", "close", "volume"]]
+        o, h, l, c, v = xr["open"], xr["high"], xr["low"], xr["close"], xr["volume"]
+
+        match volume_adjustment_method:
+            case "normal":
+                vi = v / v.shift(1)
+            case "entropy":
+                vi = v / sma(v, period)
+            case _:
+                raise ValueError(f"Invalid volume adjustment method: {volume_adjustment_method}")
+
+    else:
+        check_frame_columns(data, "open", "high", "low", "close")
+        xr = data[["open", "high", "low", "close"]]
+        o, h, l, c = xr["open"], xr["high"], xr["low"], xr["close"]
+        vi = 1
+
+    c1 = c.shift(1)
+    oi = np.log(o) - np.log(c1)
+    ci = np.log(c) - np.log(o)
+    di = np.log(l) - np.log(o)
+    ui = np.log(h) - np.log(o)
+
+    match method:
+        case "t":
+            # - true range
+            vol = sma(vi * pd.concat((abs(h - l), abs(h - c1), abs(l - c1)), axis=1).max(axis=1), period)
+            return (100 * vol / c) if percentage else vol
+
+        case "te":
+            # - true range with ema smoothing
+            vol = ema(vi * pd.concat((abs(h - l), abs(h - c1), abs(l - c1)), axis=1).max(axis=1), period)
+            return (100 * vol / c) if percentage else vol
+
+        case "c":
+            # - classical
+            m = sma(oi + ci, period)
+            vol = sma(vi * ((oi + ci) - m) ** 2, period)
+
+        case "p":
+            # - parkinson
+            vol = sma(vi * (ui - di) ** 2, period) / (4 * np.log(2))
+
+        case "rs":
+            # - rogers-satchell
+            vol = sma(vi * (ui * (ui - ci) + di * (di - ci)), period)
+
+        case "gk":
+            # - garman-klass
+            vol = sma(vi * ((ui - di) ** 2 - (4 * np.log(2) - 2) * ci**2), period)
+
+        case "yz":
+            # - yang-zhang
+            k = 0.34 / (1.34 + (period + 1) / (period - 1))
+            vol = sma(
+                vi * (ci - sma(ci, period)) ** 2
+                + k * (oi - sma(oi, period)) ** 2
+                + (1 - k) * (ui * (ui - ci) + di * (di - ci)),
+                period,
+            )
+        case _:
+            raise ValueError(f"Invalid method: {method} only t, te, c, p, rs, gk, yz are supported")
+    return 100 * vol**0.5 if percentage else vol**0.5 * c
+
+
+def fisher_transform(x: pd.Series, period: int) -> pd.Series:
+    """
+    Fisher transform of a timeseries
+    """
+    r = x.rolling(window=period)
+    xp = 2 * (r.median() - r.min()) / (r.max() - r.min()) - 1
+    return 0.5 * np.log((1 + xp) / (1 - xp))
+
+
+def wvf(
+    df: pd.DataFrame, pd_period=22, bbl=20, mult=2.0, lb=50, ph=0.85, pl=1.01, bbl_mean="sma", use_high=False
+) -> pd.DataFrame:
+    """
+    Williams VIX Fix (WVF) indicator implementation.
+    It aims to replicate the behavior of the CBOE Volatility Index (VIX) for instruments that lack options markets, such as individual stocks or ETFs.
+    The WVF measures price volatility over the past 22 days, making it a proxy for short-term market fear or complacency.
+
+    - The formula for the Williams VIX Fix is:
+        - WVF = [(Highest Close over 22 days – Current Low) / Highest Close over 22 days] × 100
+        - This calculation normalizes recent price drops relative to recent highs, producing a percentage value that reflects current volatility levels.
+    - A high reading means investor sentiment is one of increased fear, while low readings are associated with low-volatility conditions (and market tops)
+
+    Examples:
+    - - - - -
+
+    ```python
+    wvf_l = wvf(data, bbl=25, use_high=False, bbl_mean="kama", mult=3, ph=0.90, pl=1.01)
+    wvf_h = wvf(data, bbl=25, use_high=True, bbl_mean="kama", mult=3, ph=0.90, pl=1.01)
+
+    # - plotting
+    LookingGlass([
+        data,
+    ], {'V': [
+        'bars', '#404040', wvf_l.wvf * (~wvf_l.signal), 'bars', '#ff0000', wvf_l.wvf * wvf_l.signal,
+        'bars', '#404040', wvf_h.wvf * (~wvf_h.signal), 'bars', '#00ff00', wvf_h.wvf * wvf_h.signal,
+    ]}, backend='mpl'
+    ).look('2025-06-01', '2025-08-01').hover(h=600, w=1000)
+    ```
+
+    Parameters:
+    -----------
+    df : pandas.DataFrame
+        DataFrame with OHLC data (must have 'close', 'low' columns)
+    pd_period : int, default 22
+        LookBack Period for Standard Deviation High
+    bbl : int, default 20
+        Bollinger Band Length
+    mult : float, default 2.0
+        Bollinger Band Standard Deviation multiplier
+    lb : int, default 50
+        Look Back Period for Percentile High
+    ph : float, default 0.85
+        Highest Percentile (0.85=85%, 0.90=90%, 0.95=95%, 0.99=99%)
+    pl : float, default 1.01
+        Lowest Percentile (1.01=99%, 1.05=95%, 1.10=90%)
+    bbl_mean : str, default "sma"
+        Mean function for Bollinger Bands (e.g., "sma", "kama", "ema")
+    use_high : bool, default False
+        Use high instead of low for calculation (for top identification)
+
+    Returns:
+    --------
+    pandas.DataFrame with columns:
+        - wvf: Williams VIX Fix values
+        - m: SMA of WVF (Bollinger Band middle line)
+        - u: Upper Bollinger Band
+        - b: Lower Bollinger Band
+        - range_high: Percentile-based high threshold
+        - rangeLow: Percentile-based low threshold
+        - signal: Boolean signal when WVF >= upperBand or WVF >= rangeHigh
+    """
+    # - core WVF: ((highest(close, pd_period) - low) / highest(close, pd_period)) * 100
+    if use_high:
+        check_frame_columns(df, "close", "high")
+        lowest_close = df["close"].rolling(window=pd_period).min()
+        wvf_values = ((lowest_close - df["high"]) / lowest_close) * 100
+    else:
+        check_frame_columns(df, "close", "low")
+        highest_close = df["close"].rolling(window=pd_period).max()
+        wvf_values = ((highest_close - df["low"]) / highest_close) * 100
+
+    # - bollinger bands around WVF
+    m_line = smooth(wvf_values, bbl_mean, bbl)
+    s_dev = wvf_values.rolling(window=bbl).std() * mult
+    u_band = m_line + s_dev
+    l_band = m_line - s_dev
+
+    # - percentile-based thresholds
+    range_h = wvf_values.rolling(window=lb).max() * (pl if use_high else ph)
+    range_l = wvf_values.rolling(window=lb).min() * (ph if use_high else pl)
+
+    # - signal generation
+    if use_high:
+        signal = (wvf_values <= l_band) | (wvf_values <= range_l)
+    else:
+        signal = (wvf_values >= u_band) | (wvf_values >= range_h)
+
+    # Create result DataFrame
+    result = pd.DataFrame(
+        {
+            "wvf": wvf_values,
+            "m": m_line,  # middle line
+            "u": u_band,  # upper band
+            "b": l_band,  # lower band
+            "range_high": range_h,  # upper percentile band
+            "range_low": range_l,  # lower percentile band
+            "signal": signal,
+        },
+        index=df.index,
+    )
+
+    return result
