@@ -1,10 +1,11 @@
-from typing import Callable, Dict, Iterable, Literal, Optional, Union, List
 from datetime import timedelta
-import pandas as pd
-import numpy as np
+from typing import Any, Callable, Dict, Iterable, List, Literal, Optional, Set, Union
 
+import numpy as np
+import pandas as pd
 from numpy.lib.stride_tricks import as_strided as stride
 
+from qubx.core.series import OHLCV
 from qubx.utils.misc import Struct
 
 
@@ -22,7 +23,10 @@ def check_frame_columns(x, *args):
 
 
 def rolling_forward_test_split(
-    x: pd.Series | pd.DataFrame, training_period: int, test_period: int, units: str | None = None
+    x: pd.Series | pd.DataFrame,
+    training_period: int,
+    test_period: int,
+    units: str | None = None,
 ):
     """
     Split data into training and testing **rolling** periods.
@@ -50,7 +54,7 @@ def rolling_forward_test_split(
     :return:
     """
     # unit formats from pd.TimeDelta and formats for pd.resample
-    units_format = {"H": "H", "D": "D", "W": "W", "M": "MS", "Q": "QS", "Y": "AS"}
+    units_format = {"H": "h", "D": "d", "W": "W", "M": "MS", "Q": "QS", "Y": "AS", "MIN": "min"}
 
     if units:
         if units.upper() not in units_format:
@@ -90,11 +94,11 @@ def rolling_forward_test_split(
             yield (np.array(range(i - training_period, i)), np.array(range(i, i + test_period)))
 
 
-def generate_equal_date_ranges(start: str | pd.Timestamp, end: str | pd.Timestamp, freq: int, units: str) -> Iterable:
+def generate_equal_date_ranges(start: str | pd.Timestamp, end: str | pd.Timestamp, freq, units):
     """
     Generator for date ranges:
 
-    for s,e in generate_equal_date_ranges('2019-01-01', '2022-05-17', 1, 'Y'):
+    for s,e in generate_ranges('2019-01-01', '2022-05-17', 1, 'Y'):
         print(s, e)
     ------------------
     2019-01-01 2019-12-31
@@ -103,9 +107,13 @@ def generate_equal_date_ranges(start: str | pd.Timestamp, end: str | pd.Timestam
     2022-01-01 2022-05-17
     """
     _as_f = lambda x: pd.Timestamp(x).strftime("%Y-%m-%d")
+    if units in ["M", "Y"]:
+        offset = pd.DateOffset(years=freq) if units == "Y" else pd.DateOffset(months=freq)
+    else:
+        offset = pd.Timedelta(f"{freq}{units}")
 
-    # - for case when end - start < freq it won't got into the loop
-    b = [start, start] if _as_f(start) > _as_f(end) else [start, end]
+    if pd.Timestamp(end) - offset < pd.Timestamp(start):
+        b = [start, end]
 
     for a, b in rolling_forward_test_split(pd.Series(0, pd.date_range(start, end)), freq, freq, units=units):
         yield _as_f(a[0]), _as_f(a[-1])
@@ -460,6 +468,18 @@ def ohlc_resample(
         _cols = d.columns
         _source_tz = d.index.tz
 
+        # if we have trades
+        if all([i in _cols for i in ["price", "side", "amount"]]):
+            result = _tz_convert(d.price, resample_tz, _source_tz)
+            result = result.resample(freq).agg("ohlc")
+            result["volume"] = d.amount.resample(freq).sum()
+            result["quote_volume"] = (d.amount * d.price).resample(freq).sum()
+            result["taker_buy_volume"] = d[d.side == "buy"].amount.resample(freq).sum()
+            result["taker_buy_quote_volume"] = (
+                (d[d.side == "buy"].amount * d[d.side == "buy"].price).resample(freq).sum()
+            )
+            return result if not resample_tz else result.tz_convert(_source_tz)
+
         # if we have bid/ask frame
         if "ask" in _cols and "bid" in _cols:
             # if sizes are presented we can calc vmpt if needed
@@ -531,3 +551,117 @@ def shift_series(
     f0 = pd.Timedelta(forward if forward is not None else 0)
     n_sigs.index = n_sigs.index + f0 + pd.DateOffset(days=days, hours=hours, minutes=minutes, seconds=seconds)
     return n_sigs
+
+
+def _frame_to_str(data: pd.DataFrame | pd.Series, name: str, start=4, end=4, time_info=True) -> str:
+    r = ""
+    if isinstance(data, (pd.DataFrame, pd.Series)):
+        t_info = f"{len(data)} records"
+        if time_info:
+            t_info += f" | {data.index[0]}:{data.index[-1]}"
+        hdr = f"- - -({name} {t_info} records)- - -"
+        sep = " -" * 50
+        r += hdr[: len(sep)] + "\n"
+        r += data.head(start).to_string(header=True) + "\n"
+        if start < len(data) and end > 0:
+            r += "\t. . . . . . \n"
+            _s = data.tail(end).to_string(header=True)
+            r += "\n".join(_s.split("\n")[2:]) + "\n"
+    else:
+        r = str(data)
+    return r
+
+
+class OhlcDict(dict):
+    """
+    Extended dictionary with method to perform resampling on OHLCV (Open, High, Low, Close, Volume) data.
+
+    Example:
+    -------
+    >>> index=pd.date_range('2020-01-01', periods=10, freq='15min')
+        nc = np.random.rand(10).cumsum()
+        d = OhlcDict({
+            "A": pd.DataFrame({"open": nc, "high": nc,"low": nc,"close": nc }, index=index),
+            "B": pd.DataFrame({"open": nc, "high": nc,"low": nc,"close": nc }, index=index),
+        })
+        print(d.close)
+        print(d("1h").close)
+        # - just show full info about this dict
+        print(str(d))
+
+        # - get item by keys
+        print(d["A", "B"].close)
+    """
+
+    _orig: dict[str, pd.DataFrame | pd.Series | OHLCV]
+    _fields: Set[str]
+
+    def __init__(self, orig: dict[str, pd.DataFrame | pd.Series | OHLCV]):
+        self._orig = orig
+        _o_copy = {}
+        _lst = []
+        if isinstance(orig, dict):
+            for k, o in orig.items():
+                if not isinstance(o, (pd.DataFrame | pd.Series | OHLCV)):
+                    raise ValueError(
+                        f"All values in the dictionary must be pandas Series, DataFrames or OHLCV, but {k} is {type(o)}"
+                    )
+
+                if isinstance(o, OHLCV):
+                    o = o.pd()
+
+                # - skip empty data
+                if not o.empty:
+                    _o_copy[k] = o
+                    _lst.extend(o.columns.values)
+
+        self._fields = set(_lst)
+        super().__init__(_o_copy)
+
+    def __call__(self, *args: Any, **kwds: Any) -> Any:
+        if args:
+            try:
+                return OhlcDict(ohlc_resample(self, pd.Timedelta(args[0]), **kwds))
+            except Exception as e:
+                raise ValueError(str(e))
+        return self
+
+    def __getattribute__(self, name: str) -> Any:
+        if name != "_fields":
+            if name in self._fields:
+                return retain_columns_and_join(self, name)
+        return super().__getattribute__(name)
+
+    def display(self, heads=-1, tails=0) -> str:
+        r = ""
+        _h, _t = (len(self), 0) if heads == -1 else (heads, tails)
+        for k, v in self.items():
+            r += _frame_to_str(v, name=k, start=_h, end=_t) + "\n"
+        return r
+
+    def __str__(self) -> str:
+        return self.display(3, 3)
+
+    def __repr__(self) -> str:
+        return self.display(3, 3)
+
+    def __reduce__(self):
+        """
+        For joblib Parallel compatibility - defines how to pickle the object
+        """
+        # Return the class, constructor arguments, and additional state
+        data = {k: v for k, v in self.items()}
+        return (self.__class__, (data,), {"_orig": self._orig})
+
+    def __setstate__(self, state):
+        """Restore object from pickle state"""
+        # Recreate the object using the constructor
+        self.__init__(state["_orig"])
+
+    def __getitem__(self, key: str | list | tuple) -> "pd.DataFrame | pd.Series | OhlcDict":
+        match key:
+            case str():
+                return super().__getitem__(key)
+            case list() | tuple():
+                return OhlcDict({k: self[k] for k in key})
+        raise KeyError(f"Unknown key: {key}")
